!
!  You can optain NWS Hourly Surface Wxx data from
!  http://www.ncdc.noaa.gov/most-popular-data#dsi-3505
!  1) Select ISD/CDO
!  2) Read and then select 'I agree with these terms (continue)'
!  3) I made two retrevials: 'Simplified' and 'Advanced'
!     'Simplified': a) I selected 'United States' then 'Continue'
!                   b) I selected 'State or Province' and 'Selected Stations in State'
!                   c) I selected the station I wanted then 'continue'
!                   d) I selected the time period; 'only on hour' and then 'continue'
!                   e) I then looked at whether any data was there to retreive.
!                   f) If data was available, I entered my Email and 'Submit Request'

!      'Advanced':  a) Country = 'United States' and 
!                      Output = 'Test File'
!                   b)-c) above
!                   d) I selected (hold Ctrl key down)
!                      Sky Contion Observation
!                      Sky Condition Observation #2
!                      SKy Cover Layer
!                      Sky Cover Summary and State
!                  This Second Retreival was to get the cloud cover in tenths!
!                  Everything else that I needed to put in to CD-144 format
!                  is contained in the Simplified Format
!
!
!     January 21, 2012 Updated program.
!     I decided that it was too much trouble to edit the CDO-data to insure
!     that the cloud-data and simple-data start and stop on the same day/hour.
!     So, I altered this program to solve this problem for me.
!
      character*1  atotal,aopaque,aclouds(4,12,31,0:24,2)
                                     !store clouds(*,1) = total
                                     !      clouds(*,2) = opaque
                                     ! where * is month, day, hour
                                     !assume we only process one year at a time
      character*2   aWDIR,aSPD
      character*3   aCLG,aTT,aDwpt
      character*4   aSLP,aSTP
      character*50  filename
      character*340 card
      
      integer total,opaque,iclouds(4,12,31,0:24,2)                                    
                                     
      integer  offset  !number of hours needed to be subtracted from GMT to convert to LST                                     
                                     
      data offset/6/     !number of hours to subtract form GMT to convert time to LST
      
      !fill clouds with missing data values
      do i=1,4
         do m=1,12
            do id=1,31
               do ihr=0,23
               aclouds(i,m,id,ihr,1) = ' '
               aclouds(i,m,id,ihr,2) = ' '
               iclouds(i,m,id,ihr,1) = -99
               iclouds(i,m,id,ihr,2) = -99
               end do
            end do
         end do
      end do
      
      open(20,file='CD144.txt')     
      
10    write(*,*) 'Name of Cloud data file: '
      read(*,*) filename
      open(10,file=filename,status='old',err=10)
      !read past headers
      read(10,*)
      read(10,*)
 
      istart = 0
15    continue
      read(10,1000,end=20) iyear,imonth,iday,ihour,itotal,iopaque 
1000  format(t47,3i2,i3,t300,2i3)

      if( istart.eq.0 ) then
      iyrBEGIN = iyear
      istart = 1
      end if

      !cloud amounts are in octants, convert to tenths
      if( itotal.ge.0 .and. itotal.le.8 ) then
      value = 10.0*float(itotal)/8.0
      total = NINT(value)
      elseif(itotal.eq.8) then
      total = 10
      else
      total = 99
      end if
      
      if( iopaque.ge.0 .and. iopaque.le.8 ) then
      value = 10.0*float(iopaque)/8.0
      opaque = NINT(value)
      elseif(iopaque.eq.8) then
      opaque = 10
      else
      opaque = 99
      end if
      
      iyr = iyear - iyrBEGIN + 1
      iclouds(iyr,imonth,iday,ihour,1) = total
      iclouds(iyr,imonth,iday,ihour,2) = opaque
      
      !convert total and opaque to a1 formats
      if(total.le.9) then 
      write(atotal,1005) total
1005  format(i1)
      elseif( total.eq.10 ) then
      atotal = '-'
      elseif( total.gt.10 ) then
      atotal = ' '
      end if
      
      if(opaque.le.9) then 
      write(aopaque,1005) opaque
      elseif( opaque.eq.10 ) then
      aopaque = '-'
      elseif( opaque.gt.10 ) then
      aopaque = ' '
      end if
      
      aclouds(iyr,imonth,iday,ihour,1) = atotal
      aclouds(iyr,imonth,iday,ihour,2) = aopaque            
     
      go to 15
20    continue      
      close (10)
          
25    Write(*,*) 'Name of Simplified data file: '
      Read(*,*) filename
      
      open(10,file=filename,status='old',err=25)
      read(10,*)                          !read past header

30    continue

      read(10,2000,end=500) card
2000  format(a340)  
    
      read(card,2010) iWBAN,iyear,imonth,iday,ihour,iminute
2010  format(7x,i5,1x,i4,4i2)

      call PullMet(card,aWDIR,aSpd,aCLG,aTT,aDwpt,aSLP,aSTP)
      
      !convert year to two-digits
      if(iYear.ge.2000) then
      iYear = iYear-2000
      else
      iYear = iYear-1900
      end if
     
      iyr = iyear - iyrBEGIN + 1
      atotal  = aclouds(iyr,imonth,iday,ihour,1)
      aopaque = aclouds(iyr,imonth,iday,ihour,2)
          
      call GMT2LST(offset,iyear,imonth,iday,ihour,iyrout,imnout,idyout,ihrout)
       
      itest = iclouds(iyr,imonth,iday,ihour,1)
      if( itest.eq.-99 ) go to 30     !we never read any cloud data in for this hour
      
      write(20,3000) iWBAN,iyrout,imnout,idyout,ihrout,aCLG,aSLP,aDwpt,aWDIR,aSpd,aSTP,aTT,atotal,aopaque
3000  format(i5,4i2,a3,t32,a4,a3,a2,a2,a4,a3,t56,a1,t79,a1)

      go to 30
 500  continue
      stop
      end
      
      subroutine PullMet(card,aWDIR,aSpd,aCLG,aTT,aDwpt,aSLP,aSTP) 
 
      character*2  aWDIR,aSPD
      character*3  WDIR,CLG,aCLG,Spd,aTT,aDwpt
      character*4  aSLP,aSTP
      character*6  SLP,STP
      character*340 card
      
      integer lbound(36),ubound(36)
      
      !set all output to missing initially
      aWDIR = '   '
      aSpd  = '  '
      aCLG  = '   '
      aTT   = '   '
      aDwpt = '   '
      aSLP  = '    '
      aSTP  = '    '
      
      do i=1,36
      lbound(i) = 5 + (i-1)*10
      ubound(i) = lbound(i) + 9
      if( i.eq.36 ) ubound(i) = 360
      end do
      
      read(card,1000) WDIR,Spd,CLG,aTT,aDwpt,SLP,STP
1000  format(t27,a3,1x,a3,t39,a3,t85,a3,2x,a3,t94,a6,7x,a6)

      if(WDIR.eq.'***') then  !*** is missing data  
      aWDIR = '  '
      else
      read(WDIR,2010) iWDIR
2010  format(i3)   
      !convert to nearest 10-degrees
      call ConvertWDIR(lbound,ubound,iWDIR,ivalueout) 
      iWDIR = ivalueout
      write(aWDIR,2015) iWDIR
2015  format(i2)      
      end if
      
      !******Special Patch********
      !      Set WDIR to 270 degrees
      !Take this out if you really want the true wind directions
       
      aWDIR = '27'
      
      !****************************
      
      if(Spd.eq.'***') then
      aSpd = '  '
      else
      read(SPD,2020) iSpd
2020  format(i3)      
      !convert miles per hour wind spd to knots
      SpdKnts = 0.8689778*iSpd
      iSpdKnts = NINT(SpdKnts)
         if( iSpdKnts.gt.99 ) then
         aSpd = '--'
         else
         write(aSpd,2015) iSpdKnts
         end if         
      end if
      
      if(CLG.eq.'***') then
      aCLG = '   '
      elseif( CLG.eq.'722' ) then
      !if CLG = 722 (unlimited) set CLG to --- else set to indicated value
      aCLG = '---'
      else
      aCLG = CLG
      end if
      
      if(aTT.eq.'***') then
      aTT = '   '
      end if
      
      if(aDwpt.eq.'***' ) then
      aDwpt = '   '
      end if
      
      if(SLP.eq.'******') then
      aSLP = '    '
      else
      read(SLP,2025) SLPmb
2025  format(f6.1)                  
      !convert Stations Pressure
         SLPmb = 10*SLPmb
         if(SLPmb.lt.1000.0 ) then
         iSLP = SLPmb
         else
         iSLP = SLPmb - 1000
         end if
         write(aSLP,2030) iSLP
2030     format(i4.4)  
      end if
      
      if(STP.eq.'******') then
      aSTP = '    '
      else
      read(STP,2025) STPmb
      !convert Stations Pressure to inches of Hg
      STPinches = 29.92*STPmb/1013.2034778
      STPinches = 100.0*STPinches
      iSTP = NINT(STPinches)
      write(aSTP,2030) iSTP
      end if
      
      return
      end

      subroutine ConvertWDIR(lbound,ubound,ivaluein,ivalueout)
      integer lbound(36),ubound(36)
      
      if(ivaluein.le.4) then
      ivalueout = 1
      go to 50
      end if
      
      do i=1,36
      if(ivaluein.ge.lbound(i) .and. ivaluein.le.ubound(i)) then
      ivalueout = i
      go to 50
      end if
      end do
      
      !if we get here someting went wrong
      ivalueout = 99
      
50    continue      
      return
      end
      
      subroutine GMT2LST(offset,iyear,imonth,iday,ihour,iyrout,imnout,idyout,ihrout)
      integer offset
      dimension ndays(12)
      data ndays/31,28,31,30,31,30,31,31,30,31,30,31/
      ! convert GMT to LST
      ! offset = number of hours to subtract to get to LST in USA
      
      leap = MOD(iyear,4)
      if( leap.eq.0 ) then
      leap = 1
      else
      leap = 0
      end if
      
      ! leap   = 0 for a normal year
      !        = 1 for a leap year where Feb has 29 days
      
      iyrout = iyear
      imnout = imonth
      idyout = iday
      
      ihrout = ihour - offset
      if( ihrout.ge.0 ) then
      go to 50
      end if
      
      ! we have gone back into a previous day
      ihrout = 24 + ihrout
      idyout = iday - 1
      if( idyout.gt.0 ) then
      go to 50
      end if
      
      !we have gone back into a previous month
      if( imonth.ge.2 ) then
      imnout = imonth - 1
      else
      imnout = 12      !this is December of the previous year
      iyrout = iyear - 1
      end if
      
      if( imnout.eq.2 .and. leap.eq.1 )then
      idyout = ndays(imnout) + 1 + idyout
      else
      idyout = ndays(imnout) + idyout
      end if     

50    continue
      return
      end
      
     
      