!
!  When I Processed the Project Prairie Grass data, I decided to set up
!  the meteorology for use by AERMOD, and then convert the OBK to a Pasquill
!  stability category for use by ISCST3.
!
!  Since AERMET uses 3-hr averages for certain variables, I had to run all 
!  hours. But I do not need to simulate concentrations for all hours, only
!  for those hours for which I have obsrvations.
!
!  This program reads the AERMET files surf and prof created for use by AERMOD, and:
!  1) stores off those hours (in AERMOD's format) that we wish to compute
!     concentrations.
!  2) I also use the AERMET OBK value to compute a Pasquill stability category
!     and then list out data for use by ISCST3.
!
      character*30 filename
      character*100 header
      character*165 cardsurf,cardprof
      
      data ifirst/0/
      
      NoProf = 0
      NoSurf = 0
      NoOBK  = 0
      
      open(55,file='CutmetSummary.txt')
      
10    write(*,*) ' Dates filename: '
      read(*,*) filename
      open(20,file=filename,status='old',err=10)
      
15    write(*,*) ' AERMET surf input ALL hours filename: '
      read(*,*) filename
      open(30,file=filename,status='old',err=15)
      !read header of surf file      
      read(30,1000) header
1000  format(a100)
      
      open(35,file='cutsurf.txt')
      !write header to cut surf file being developed
      write(35,1000) header
     
20    write(*,*) ' AERMET prof input ALL hours filename: '
      read(*,*) filename
      open(40,file=filename,status='old',err=20)     
       
      open(45,file='cutprof.txt')
      ! AERMET's Prof-file has no header
      
      !open cut file for ISCST3 met being developed
      open(50,file='cutiscst3.txt')
      ! I will need to construct a header, but first I need to 
      ! know what year we are processing.
       
30    continue
      read(20,1050,end=500) iyear,imonth,iday,ihour
1050  format(i2,3i3) 
      rewind (30)     
      !past read header of surf file      
      read(30,1000) header
      
40    continue
      !search for match in date
      read(30,2000,end=400) cardsurf
2000  format(a165)      
      read(cardsurf,2010) iyr,imn,idy,ihr
2010  format(i2,2i3,4x,i3)

         if(ifirst.eq.0 ) then
         ifirst = 1
         !I need the year to create a header for the ISC cut file
         !now I have it, so we fetch the SRC and UA station numbers from
         !the header, and create a header for the ISC cut file.
         !**** I set the two met IDs to be the same, because I
         !     could not run ISCST3 successfully with two different
         !     met IDs. Do not know why??????
         read(header,2020) id1,id1
2020     format(t41,i5,t58,i5)
         write(50,2030) id1,iyr,id2,iyr
2030     format(t8,i5,t20,i2,t29,i5,t44,i2)                         
         end if
      
      if( iyear.eq.iyr .and. imonth.eq.imn .and. iday.eq.idy .and. ihour.eq.ihr ) then
      !If we do not sufficient meteorlogy, we cannot compute concentrations
      read(cardsurf,2035) OBK
2035  format(t56,f9.1)
         if(OBK.le.-99999.0)then
         go to 300
         end if
         
         
         !I am not sure that I have surf data for every prof and visa versa
         !So, I am going to now search the prof file for this date
         call searchprof(iyear,imonth,iday,ihour,cardprof,ierr)
         if( ierr.eq.0 ) then
         write(35,2000) cardsurf
         write(45,2000) cardprof
         else
         !We have no prof data for this Date
         NoProf = NoProf + 1
         write(55,3000) NoProf,iyear,imonth,iday,ihour
3000     format(' NoProf: ',i3,' No Prof data for: ',4i3)         
         go to 30
         end if
      
      !It is here that we fabricate the meteorology needed by ISCST3
      !read Surf data and retreive HFLUX, mixing heights, OBK, zo, wind speed and temperature
      read(cardsurf,2040) HFLUX,Zi1,Zi2,OBK,zo,WS,Temp
2040  format(t16,f7.1,t44,2f6.0,f9.1,f8.4,t88,f7.2,t109,f7.1)

         if(HFLUX.gt.0.0 ) then
         Zi2 = Zi1
         else
         Zi1 = Zi2
         end if
         
      WDIR = 90.0
      call OBK2KST(OBK,HFLUX,zo,KST,ierror) 
      write(50,2050) iyear,imonth,iday,ihour,WDIR,WS,Temp,KST,Zi1,Zi2
2050  format(4i2,2f9.4,f6.1,i2,2f7.1)
      go to 30    !fetch a new date to look for
      else      
      go to 40    !fetch a new hours met data
      end if
      
      go to 500
      
300   continue
      !We have no AERMET OBK value
      NoOBK = NoOBK + 1
      write(55,3025) NoOBK,iyear,imonth,iday,ihour
3025  format(' NoOBK: ',i3,' No Aermet OBK data for: ',4i3)       
      go to 30              
      
400   continue

      if( iyear.ne.0 .and. imonth.ne.0 ) then
      !we have no surf data for this Date   
      NoSurf = NoSurf + 1
      write(55,3050) NoSurf,iyear,imonth,iday,ihour
3050  format(' NoSurf: ',i3,' No Surf data for: ',4i3)       
      end if
      
      go to 30   
      
500   continue
      stop
      end
      
      subroutine searchprof(iyr,imn,idy,ihr,cardprof,ierr)
      character*165 cardprof
      !this is brutal and slow but it insures that if we have 
      !a proper surf we match it up with a proper prof
      rewind(40)
      ierr = -1
10    continue
      read(40,1000,end=50) cardprof      
1000  format(a165)      
      read(cardprof,1050) iyear,imonth,iday,ihour
1050  format(i2,3i3)
      if( iyear.eq.iyr .and. imonth.eq.imn .and. iday.eq.idy .and. ihour.eq.ihr ) then
      ierr = 0
      go to 50
      end if
      go to 10
50    continue      
      return
      end
      
      
   !*******************************************************************************
      subroutine OBK2KST ( OBK, HFLUX, zo, KST, ierror )
      
      !     VERSION Jan 20, 2013

!     This routine takes P-G class (Kst) & surface roughlness length (zo) 
!     and returns Monin-Obukhov length (Obk), friction velocity (ustar) and sensible heat
!     flux (W/m^2).
!
      DIMENSION A(6),B(6),OBKmid(6), OBKBoundary(5)

      DATA A/-0.0875,-0.03849,-0.00807,0.0,0.00807,0.03849/ 
      DATA B/-0.1029,-0.1714,-0.3049,0.0,-0.3049,-0.1714/
!     COEF. FOR OBUKHOV LENGTH GIVEN zo AND P-G CATEGORY
!     AS REF. IN IRWIN(1979), PROCEEDINGS 4TH AMS SYMP. ON TURB., DIFFU., AND
!     AIR POLLU.; RENO, NV.      
!     Added DD = D-Day and DN = D-Night     

 
!     Passed Input:
!     Obk   = Monin-Obukhov Length (m)
!     HFLUX = AERMET computed heat flux (W/m^2) I use this for D-day or D-night
!     zo    = surface roughness lengh (m)

!     Passed Output:
!     Kst         = Pasquill stability category (1=A, 2=B, 3=C, 4=DD, 5=DN, 6=E, 7=F)

      ierror = 0
!
!     I compute the midpoint OBK for each stability category
      do kst=1,6
         if(kst.eq.4 ) then
         OBKmid(kst) = 0.0
         else
         OBKmid(kst) = A(Kst)*zo**B(Kst)
         end if
      end do
     
      !Now find the boundary between each Pasquill category
      do kst=1,5
      OBKBoundary(kst) = 0.5*(OBKmid(kst) + OBKmid(kst+1))
      end do
     
      !use HFLUX to determine whether to search for a negative or positive L-value
      OvrOBK = 1.0/OBK
      if( HFLUX.gt.0.0 ) then
      
      !L is < zero
        KST = -99
        isearch = 0
        do i=1,3
           if(OvrOBK.lt.OBKBoundary(i) .and. isearch.eq.0 ) then
           KST = i
           isearch = 1
           end if
        end do
        
        !KST = 99 then it must be D-day
        if( KST.eq.-99 ) then
        KST = 4
        end if
        
      endif
      
      if( HFLUX.lt.0.0 ) then
      !L is > 0
        KST = -99
        isearch = 0
        do i=5,4,-1
           if(OvrOBK.gt.OBKBoundary(i) .and. isearch.eq.0 ) then
           KST = i+1
           isearch = 1
           end if
        end do
        
        !KST = 99 must be F stability
        if( KST.eq.-99 ) then
        KST = 4
        end if

      end if
      
      !Do some error checking
      If( HFLUX.gt.0.0 .and. KST.gt.4 ) then
      ierror = 1
      elseif( HFLUX.lt.0.0 .and. KST.lt.4 ) then
      ierror = 2
      end if
      
      if( OBK.le.-99999.0 ) then
      !OBK is missing
      KST = -9
      end if
   
      RETURN
      END
  